package xframework.com.controller;

import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.stereotype.Controller;
import org.springframework.ui.ModelMap;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;

import xdataset.XDataSet;
import xframework.com.service.ExampleService;

@Controller
public class SimpleXDataSetController {
	@RequestMapping(value="/mytest.do", method=RequestMethod.GET)
	public String test(){
		System.out.println("####### Hello this is MyController 클래스 ");
		return "test.jsp";
	}
	
	
	@Resource(name = "exampleService")
	private ExampleService exampleService;
	
	private Log log = LogFactory.getLog(this.getClass());
	
	/**
	 * xframe 조회  (기본방식) 
	 * @param request 스프링에서 제공해주는 HttpServletRequest
	 * @param response 스프링에서 제공해주는 HttpServletResponse
	 * @param model 스프링에서 제공해주는 ModelMap
	 */
	@RequestMapping(value = "/sample/xdatasetSelect.do")
	public void select(HttpServletRequest request, HttpServletResponse response, ModelMap model) {
		log.info("xframe 조회  (기본방식)");
		
		XDataSet xDataSet 		= null;
		String strEmpAge 			= null;
		Map<String, Object> element = null;
		int nRow 					= 0;
		
		try {
			// XDataSet오브젝트 생성
			xDataSet = new XDataSet(request, response);
			
			// XDataSet오브젝트에 포함되어 있는 부가 메타 정보를 로깅
			log.info("ScreenNo : " + xDataSet.getScreenNo());
			log.info("IP : " + xDataSet.getTerminalIpAddress());
			log.info("Map ID : " + xDataSet.getTransactionMapId());
			log.info("User Header : " + xDataSet.getUserHeader());					
			
			// XDataSet오브젝트로부터 직원 정보 조회 기준인 직원 이름 정보를 얻어온다.
			// 이 정보는 "TR_SELECT" 트랜잭션의 입력 데이터 셋인 "DS_EMP_AGE"의 "EMP_AGE" 칼럼에서 정보를 가져온다.
			strEmpAge = xDataSet.getData("DS_REQ", "EMP_AGE", 0);	// ds_input 데이터셋의 첫번째줄의 EMP_NAME 값 반환	
			
			// 직원 나이로 DB검색
			List<Map<String,Object>> empInfoList = exampleService.selectMapListByStr(strEmpAge);
			Iterator<Map<String, Object>> iterator = empInfoList.iterator();
	
			// DB검색 결과와 출력 데이터셋을 XDataSet 오브젝트에 추가
			while (iterator.hasNext()){
				element = iterator.next();
				xDataSet.setData("DS_RES", "EMP_NAME", nRow, String.valueOf(element.get("EMP_NAME")));
				xDataSet.setData("DS_RES", "EMP_NO", nRow, String.valueOf(element.get("EMP_NO")));	
				xDataSet.setData("DS_RES", "EMP_AGE", nRow, String.valueOf(element.get("EMP_AGE")));	
				xDataSet.setData("DS_RES", "DEPT_CODE", nRow, String.valueOf(element.get("DEPT_CODE")));	
				xDataSet.setData("DS_RES", "TITLE_CODE", nRow, String.valueOf(element.get("TITLE_CODE")));	
				xDataSet.setData("DS_RES", "MAIL_ADDR", nRow,String.valueOf(element.get("MAIL_ADDR")));
				
			    nRow++;
			}
			
			// 유저헤더부 세팅시 아래와 같이 호출
			xDataSet.setUserHeader("user_header_msg");
			// 일반메시지 세팅시 아래와 같이 호출
			xDataSet.setMessage("200", "성공했습니다");
			
			xDataSet.returnData();	// response로 데이터 반환처리함		
			
		} catch (Exception e) {		
			log.error("select = " + e);
			try {
				if(xDataSet != null) {
					// 로직 처리중 Exception이 발생할 경우, XDataSet오브젝트에 에러 코드와 에러 메시지를 설정하고 송신
					xDataSet.setErrorMessage("-100", e.toString());
					xDataSet.returnData();	//response로 데이터 반환처리	
				}
			} catch (Exception ex) {
				log.error(ex);
			}
		}
	}
	
	/**
	 * xframe 업데이트  (제공 유틸 사용)
	 * 여러건의 레코드 데이터를 갱신하는 샘플
	 * BeanNameViewResolver 를 사용하는 방식
	 * @param request 스프링에서 제공해주는 HttpServletRequest
	 * @param response 스프링에서 제공해주는 HttpServletResponse
	 * @param model 스프링에서 제공해주는 ModelMap
	 */
	@RequestMapping(value = "/sample/xdatasetUpdate.do")
	public void update(HttpServletRequest request, HttpServletResponse response, ModelMap model) {
		log.info("xframe 업데이트  (제공 유틸 사용)");
		
		XDataSet xDataSet 		= null;
		int recordCount 		  = 0;
		int nRow 				  = 0;
		int result 				  = 0;
		Map<String,Object> empMap = null;
		
		try {
			// XDataSet오브젝트 생성
			xDataSet = new XDataSet(request, response);
			
			// 데이타셋의 레코드 갯수를 구함.
			recordCount = xDataSet.getRecordCount("DS_RES");
			
			for (nRow = 0; nRow < recordCount; nRow++) {
				empMap = new HashMap<String,Object>();
				empMap.put("EMP_NAME", xDataSet.getData("DS_RES", "EMP_NAME", nRow));
				empMap.put("EMP_NO", xDataSet.getData("DS_RES", "EMP_NO", nRow));
				empMap.put("EMP_AGE", xDataSet.getData("DS_RES", "EMP_AGE", nRow));
				empMap.put("DEPT_CODE", xDataSet.getData("DS_RES", "DEPT_CODE", nRow));
				empMap.put("TITLE_CODE", xDataSet.getData("DS_RES", "TITLE_CODE", nRow));
				empMap.put("MAIL_ADDR", xDataSet.getData("DS_RES", "MAIL_ADDR", nRow));
				
				// ROW(레코드)의 상태에 따라 서비스 처리
				if (xDataSet.isUpdateRecord("DS_RES", nRow)) {
					result = exampleService.updateMapListByMap(empMap);	
				} else if (xDataSet.isDeleteRecord("DS_RES", nRow)) {
					result = exampleService.deleteMapListByMap(empMap);	
				} else if (xDataSet.isInsertRecord("DS_RES", nRow)) {
					result = exampleService.insertMapListByMap(empMap);	
				}
			}
			
			// 유저헤더부 세팅시 아래와 같이 호출
			xDataSet.setUserHeader("user_header_msg");
			
			// 일반메세지 세팅시 아래와 같이 호출
			xDataSet.setMessage("200", "저장 완료 되었습니다.");
			
			xDataSet.returnData();	// response로 데이터 반환처리함
			
		} catch (Exception e) {		
			log.error("update = " + e);
			
			try {
				if(xDataSet != null) {
					// 에러발생시 화면으로 보낼 에러메시지를 설정
					xDataSet.setErrorMessage("-100", "데이터 저장에 실패 하였습니다.");
					xDataSet.returnData();	//response로 데이터 반환처리	
				}
			} catch (Exception ex) {
				log.error(ex);
			}
		}
	}
}